/*
 * file CcTypeBase.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 *
 * com.ibm.rational.wvcm.stp.cc.CcTypeBase
 * 
 * (C) Copyright IBM Corporation 2004, 2011.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */


package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;

/**
 * <p>Base interface for all ClearCase "type" interfaces - branch type,
 * label type, etc.  A type object controls certain aspects of the creation and
 * behavior of instances of that type.  For instance, the name of a label
 * instance is determined by it's label type.  Changing the name of a label type
 * changes the names of all instances of that type.
 * </p>
 */
public interface CcTypeBase extends CcVobResource
{
    /**
     * Scope of a ClearCase type object.
     * A type object's scope determines it's sphere of influence.  Scope is one of
     * <bl>
     * <li>None.  The scope of this type object is limited to instances in the
     * VOB in which this type resides.
     * </li>
     * <li>Local.  This type object is a VOB-local instance of a global type.
     * While it controls the behavior of instances in its local VOB, it is
     * essentially a local copy of a global type. 
     * </li>
     * <li>Global.  This type object controls behavior of instances in all VOBs
     * for which this type object's VOB serves as an "admin" VOB.  It uses
     * "local" types as proxies in VOBs other than its own.
     * </li>
     * </bl>
     * <p>
     * For more information, see the ClearCase Administrator's Guide.
     */
    enum Scope implements StpExEnumeration 
    {
        /** This is a global type. */
        GLOBAL,

        /** This is a local instance of a global type. */
        LOCAL,

        /** This is neither a global type nor a local instance. */
        NONE;
    }

    /**
     * The scope of this type object.
     */
    PropertyName<Scope> SCOPE =
        new PropertyName<Scope>(PROPERTY_NAMESPACE, "vob-type-scope");

    /**
     * Get the value of this proxy's {@link #SCOPE} property.
     * @return this type object's scope
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    Scope getScope() throws WvcmException;

    /**
     * Set the value of this proxy's {@link #SCOPE} property.
     * This property can only be set at type creation time.
     * @param scope The new scope
     */
    void setScope(Scope scope);

    /**
     * Instance constraint of a ClearCase type object.
     * A type object's instance constraint determines where the type may be applied. 
     * Some metatypes do not support all (or any) instance constraints.
     * Instance constraint is one of
     * <bl>
     * <li>None.  There are no special constraints on this metatype beyond the default behaviour
     * for the type.
     * </li>
     * <li>Element.  Type instance may only occur once per element,
     * </li>
     * </li>
     * <li>Branch.  Type instances may occur on multiple branches, but only once per branch,
     * </li>
     * </li>
     * <li>Version.  Type instance may only be attached to versions, but may be attached to 
     * as many versions as desired
     * </li>
     * </bl>
     * <p>
     * For more information, see the ClearCase Administrator's Guide.
     */
    enum InstanceConstraint implements StpExEnumeration 
    {
        /** Default constraint for this type. */
        NONE,

        /** Once per element. */
        ELEMENT,

        /** Once per branch. */
        BRANCH,
        
        /** Versions only, as many as desired. */
        VERSION;

        private InstanceConstraint() {}

        /**
         * Returns the InstanceConstraint enumerator defined by the given Object
         *
         * @param   tag  The Object for which an enumerator is sought
         *
         * @return  An InstanceConstraint object that is based on the given Object. Will
         *          be <b>null</b> if no such enumerator exists.
         */

        public static InstanceConstraint byTag(Object tag)
        {
            return StpExEnumerationBase.getByTag(InstanceConstraint.class, tag);
        }
    }
    
    /**
     * The instance constraint of this type object.
     */
    PropertyName<InstanceConstraint> INSTANCE_CONSTRAINT =
        new PropertyName<InstanceConstraint>(PROPERTY_NAMESPACE, "vob-type-instance-constraint");

    /**
     * Get the value of this proxy's {@link #INSTANCE_CONSTRAINT} property.
     * @return this type object's instance constraint
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    InstanceConstraint getInstanceConstraint() throws WvcmException;

    /**
     * Set the value of this proxy's {@link #INSTANCE_CONSTRAINT} property.
     * This property can only be set at type creation time.
     * @param constraint The new instance constraint
     */
    void setInstanceConstraint(InstanceConstraint constraint);

    /**
     * Does this type object have shared mastership? If true, instances of this
     * type can be created in any VOB replica in this type object's VOB family.
     * Otherwise, instances of this type can only be created in the VOB replica
     * that is the current master of this type object. For more information, see
     * the ClearCase Administrator's Guide.
     */
    PropertyName<Boolean> HAS_SHARED_MASTERSHIP =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "has-shared-mastership"); //$NON-NLS-1$

    /**
     * Get the value of this proxy's {@link #HAS_SHARED_MASTERSHIP} property.
     * @return true if this type object has shared mastership, else false
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    boolean getHasSharedMastership() throws WvcmException;

    /**
     * Set the value of this proxy's {@link #HAS_SHARED_MASTERSHIP} property.
     * This property can only be set at type creation time.
     * @param hasSharedMastership true if this type is to have shared mastership, else false
     */
    void setHasSharedMastership(boolean hasSharedMastership);
    
    /** Enum for the doCreateResource method */
    enum TypeCreateFlag implements StpExEnumeration {

        /**
         * Acquire eclipsing types
         */
        ACQUIRE("acquire");

        private String m_name;

        private TypeCreateFlag(String name) { m_name = name; }

        /* (non-Javadoc)
         * @see java.lang.Object#toString()
         */
        public String toString() { return m_name; }
    }
}
